/*
    Project: TCTEMP
    Version: V1.16
    File   : tc_random_header.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tc_priv.h"

//#define USE_ASCII_PASSWORD

#pragma pack(push, 1)

typedef struct
{
    TC_UINT8 rgbSalt[64];
    TC_UINT8 rgbPasswordSalt[64];
    TC_UINT8 rgbReserved[128];
    TC_KEY   keyVolume; /* 192 bytes */
    TC_UINT8 rgbPassword[64];
} TC_OLD_HEADER, *PTC_OLD_HEADER;

#pragma pack(pop)

BOOL TC_MakeRandomHeader(PTC_HEADER pHeader, PTC_PASSWORD pPassword,
  PTC_ENTROPY_POOL pEntropyPool, unsigned uHash, unsigned uCipher,
  unsigned uIterationCount, PTC_BUFFER pBuffer, TC_UINT64 ullVolumeSize)
{
    PTC_OLD_HEADER poh;
    unsigned       i;
#ifdef USE_ASCII_PASSWORD
    unsigned char  *pb;
    unsigned char  *pbEnd;
#endif

    poh = (PTC_OLD_HEADER)pHeader;

    if (!TC_HashIsValid(uHash) || !TC_CipherIsValid(uCipher)) {
        return FALSE;
    }

    pPassword->rgbPassword[TC_MAX_PASSWORD_LEN] = '\0';
    pPassword->cbPasswordSize = TC_MAX_PASSWORD_LEN;
    for (i = 256; --i; ) {
        if (!TC_DeriveKey(&poh->keyVolume,
          sizeof(TC_KEY)+sizeof(poh->rgbPassword), pEntropyPool->rgbPool,
          TC_GetEntropyPoolSize(pEntropyPool), poh->rgbPasswordSalt,
          sizeof(poh->rgbPasswordSalt), uHash, uIterationCount,
          &pBuffer->pkcs5_2_state)) {
            return FALSE;
        }
        memcpy(pPassword->rgbPassword, poh->rgbPassword, TC_MAX_PASSWORD_LEN);
#ifdef USE_ASCII_PASSWORD
        pb = pPassword->rgbPassword;
        pbEnd = pb + TC_MAX_PASSWORD_LEN;
        for (; pb != pbEnd; pb++) {
            *pb = '0' + (*pb & 0x3f);
        }
#endif
        if (!TC_KeyIsWeak(&poh->keyVolume, uCipher) &&
          !TC_PasswordIsWeak(pPassword, 256, &pBuffer->pi)) {
            break;
        }
        poh->rgbPasswordSalt[0]++;
    }
    if (i == 0) {
        return FALSE;
    }

    memset(poh->rgbSalt, '\0', 256);
    memset(poh->rgbPassword, '\0', sizeof(poh->rgbPassword));
    if (TCE_Init(&pBuffer->ctx, &poh->keyVolume, uCipher)) {
        TCE_SetIndex(&pBuffer->ctx, CONST64(0xffffFFFFffffFFFF),
          CONST64(0xffffFFFFffffFFFF));
        if (!TCE_Encrypt(&pBuffer->ctx, poh->rgbSalt, 64)) {
            TCE_Close(&pBuffer->ctx);
            return FALSE;
        }
        TCE_Close(&pBuffer->ctx);
    }

    return TC_EncryptHeader(pHeader, pPassword->rgbPassword,
      TC_MAX_PASSWORD_LEN, uHash, uCipher, pBuffer, ullVolumeSize);
}
